#!/bin/bash
#
#  YUP - "Yum Update Proxy" for SLE 10 Maintenance
#  Copyright (C) 2006 SUSE Linux Products GmbH
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#
#  This packages allows to configure an update-mirror
#  for Novell maintenance packages for SUSE Linux
#  Enterprise 10.
#
#  Please make sure, that the number of systems to serve with the
#  mirrored packages is in compliance with the number of
#  subscriptions you are allowed to exercise according to your
#  contract with Novell.
#
#  To configure network access rules (proxies, proxy
#  authentication, ...) please use YaST2 -> Network Services ->
#  Proxy.  The resulting configuration can be found in
#  /etc/sysconfig/proxy and /root/.curlrc (authentication data).
#
#  $Id: yup.sh 11 2006-09-15 06:26:54Z mge $
#

# import the update parameters
# ( YUP_CURL YUP_CURLPARAMS YUP_DEST_DIR 
#   YUP_ID YUP_PASS YUP_ID_SLED YUP_PASS_SLED 
#   YUP_ID_SLES YUP_PASS_SLES YUP_ARCH YUP_PRODUCTS )
#
. /etc/sysconfig/yup

# evaluate update parameters
#
echo "YUP_CURL       ="${YUP_CURL:="/usr/bin/curl"}
echo "YUP_CURLPARAMS ="${YUP_CURLPARAMS:="--silent --digest --remote-time"}
echo "YUP_SERVER     ="${YUP_SERVER:="update.novell.com"}
echo "YUP_DEST_DIR   ="${YUP_DEST_DIR:="/tmp/SLE10-YUP"}
echo "YUP_ARCH       ="${YUP_ARCH:="i586"}
echo "YUP_PRODUCTS   ="${YUP_PRODUCTS:="SLES10"}
#
if [ "$YUP_ID." = "." ] || [ "$YUP_PASS." = "." ] ; then
	if [ "$YUP_ID_SLED." = "." ] || [ "$YUP_PASS_SLED." = "." ] ; then
		if [ "$YUP_ID_SLES." = "." ] || [ "$YUP_PASS_SLES." = "." ] ; then
			echo "UpdateMirror not configured properly - please run: "
			echo "YaST->System->Sysconfig Editor->System->YUP"
			exit 1;
		fi
	fi
fi

# import public proxy parameters; proxy-authentication params
# are in /root/.curlrc and automatically evaluated by curl.
# ( PROXY_ENABLED HTTPS_PROXY )
#
. /etc/sysconfig/proxy

# evaluate proxy parameters
#
if [ "$PROXY_ENABLED" = "yes" ]; then
	if ! [ "$HTTPS_PROXY." = "." ]; then
		YUP_CURLPARAMS="$YUP_CURLPARAMS --proxy $HTTPS_PROXY"
	fi
fi

for useARCH in $YUP_ARCH; do
	for SLE in $YUP_PRODUCTS; do
		useID=$YUP_ID
		usePASS=$YUP_PASS
		useSERVER=$YUP_SERVER
		if [ "$SLE." = "SLES10." ]; then
			UPD="SLES10-Updates/sles-10"
			if [ "$YUP_ID_SLES." != "." ] && [ "$YUP_PASS_SLES." != "." ] ; then
				useID=$YUP_ID_SLES
				usePASS=$YUP_PASS_SLES
			fi
			if [ "$YUP_SERVER_SLES." != "." ] ; then
				useSERVER=$YUP_SERVER_SLES
			fi
		elif [ "$SLE." = "SLED10." ]; then
			UPD="SLED10-Updates/sled-10"
			if [ "$YUP_ID_SLED." != "." ] && [ "$YUP_PASS_SLED." != "." ] ; then
				useID=$YUP_ID_SLED
				usePASS=$YUP_PASS_SLED
			fi
			if [ "$YUP_SERVER_SLED." != "." ] ; then
				useSERVER=$YUP_SERVER_SLED
			fi
		elif [ "$SLE." = "SDK10." ]; then
			UPD="SLE10-SDK-Updates/sles-10"
			if [ "$YUP_ID." = "." ] || [ "$YUP_PASS." = "." ] ; then
				if [ "$YUP_ID_SLES." = "." ] || [ "$YUP_PASS_SLES." = "." ] ; then
					useID=$YUP_ID_SLED
					usePASS=$YUP_PASS_SLED
				else
					useID=$YUP_ID_SLES
					usePASS=$YUP_PASS_SLES
				fi
			fi
			if [ "$YUP_SERVER." = "." ] ; then
				if [ "$YUP_SERVER_SLES." = "." ] ; then 
					useSERVER=$YUP_SERVER_SLED
				else
					useSERVER=$YUP_SERVER_SLES
				fi
			fi
		else
			echo "Warning: SLE $SLE unknown. Proceeding anyways ..."
			# exit 1
		fi

		bn="https://${useID}:${usePASS}@${useSERVER}/repo/\$RCE/${UPD}-${useARCH}/"

		echo "Using $bn"

		mkdir -p ${YUP_DEST_DIR}/${SLE}/${useARCH}
		cd ${YUP_DEST_DIR}/${SLE}/${useARCH}

		mkdir repodata 2>/dev/null

		# Always download all the necessary metadata

		$YUP_CURL $YUP_CURLPARAMS $bn/repodata/repomd.xml.asc -o repodata/repomd.xml.asc
		$YUP_CURL $YUP_CURLPARAMS $bn/repodata/repomd.xml.key -o repodata/repomd.xml.key
		$YUP_CURL $YUP_CURLPARAMS $bn/repodata/repomd.xml -o repodata/repomd.xml
		$YUP_CURL $YUP_CURLPARAMS $bn/repodata/filelists.xml.gz -o repodata/filelists.xml.gz
		$YUP_CURL $YUP_CURLPARAMS $bn/repodata/primary.xml.gz -o repodata/primary.xml.gz
		$YUP_CURL $YUP_CURLPARAMS $bn/repodata/patches.xml -o repodata/patches.xml

		# Download all the rpms mentioned in primary.xml.gz for the YUM repo.
		zcat repodata/primary.xml.gz|grep location.href| while read line ; do
			  uri=`echo "$line"|sed -e 's/^[^"]*"//;s/".*$//;'`
			  dn=`dirname $uri`
			  test -d $dn || mkdir -p $dn
			  test -f "$uri" || $YUP_CURL $YUP_CURLPARAMS "$bn/$uri" -o "$uri"
			  echo $uri
		done

		# Download all the repo-md patches, cross check with SHA1 sum for changed ones.
		grep -E "(location.href|checksum.type..sha)" repodata/patches.xml| while read line ; do
			echo "$line"|grep "checksum.type..sha" >/dev/null
			res=$?
			if [ "$res" == "0" ]; then
				sha1=`echo "$line"|sed -e 's/^.*sha">//;s/<.checksum.*$//;'`
			else
				uri=`echo "$line"|sed -e 's/^[^"]*"//;s/".*$//;'`
				dn=`dirname $uri`
				test -d $dn || mkdir -p $dn
				nsum=`sha1sum "$uri"|sed -e 's/ .*//;'`
				if [ "x$nsum" != "x$sha1" ]; then
					echo "$uri is bad ($nsum / $sha1), reloading"
					rm -f "$uri"
				fi
				test -f "$uri" || $YUP_CURL $YUP_CURLPARAMS "$bn/$uri" -o "$uri"
				echo $uri
			fi
		done

		# Download all the patch and delta rpms mentioned in the patches.
		grep location.href repodata/patch-*.xml| while read line ; do
			  uri=`echo "$line"|sed -e 's/^[^"]*"//;s/".*$//;'`
			  dn=`dirname $uri`
			  test -d $dn || mkdir -p $dn
			  test -f "$uri" || $YUP_CURL $YUP_CURLPARAMS "$bn/$uri" -o "$uri"
			  echo $uri
		done

	done
done

#
#	Finis Africae
#

